/////////////////////////////////////////////////////////////////////////////
//
// Project:         RemoSens
//
// Name:            FirmwareUpdate.cpp
//
// Purpose:         Programming example on how to update the firmware of the
//                  ethernet/network interface-box 'RemoSens'
//
// Description:
//
// Author:          Juergen D. Geltinger
// Created:         2022-02-24
// Modified by:     Juergen D. Geltinger
// Modified:        2022-02-25
//
// Support:         support@ifd.de
//
// Acknowledgement: 'Rohde&Schwarz' and 'NRP' are registred trademarks
//                  of Rohde & Schwarz GmbH & Co KG, Munich
//
// Copyright:       IFD Ing.-Buero fuer Datentechnik, Altdorf
//
/////////////////////////////////////////////////////////////////////////////


#include <windows.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>

// Should have an installed (R&S-)VISA
#include "visa.h"



// -----------------------------------------------------------------
// main
// -----------------------------------------------------------------
int main(int iArgC, char *cpArgV[])
{
  ViSession rm;
  ViSession session;
  char      szBuf[200];
  ViStatus  lStat = 0;
  ViUInt32  lCount = 0;
  int       iRemainingTime = 60; // seconds

  // Information from the User:
  // ==========================
  char szResource[256];     // VISA Resource of RemoSens
                            // for example "TCPIP::192.168.2.108::5025::SOCKET"
  char szUpdateFile[512];   // Filename of Firmware-Update File
                            // for example "fw1.7.31.remo"
  // ==========================


  //==================================================================
  //==================================================================
  // Below this line, there is no need for a user of this application
  // to change anything of the code
  //==================================================================
  //==================================================================

  if (iArgC != 3)
  {
    fprintf(stderr, "Usage:\n");
    fprintf(stderr, "  %s  <VISA SCPI-RAW Resource>  <FW-File>\n\n", cpArgV[0]);
    fprintf(stderr, "Example\n");
    fprintf(stderr, "  %s  TCPIP::192.168.2.108::5025::SOCKET  fw1.7.31.remo\n", cpArgV[0]);
    return -1;
  }

  strncpy(szResource, cpArgV[1], sizeof(szResource));
  strncpy(szUpdateFile, cpArgV[2], sizeof(szUpdateFile));

  // Opening a VISA connection to a 'RemoSens' interface-box
  // Hint: 'RemoSens' displays its TCP/IP address on the front panel;
  //       replace the following IP address with the local one of
  //       your 'RemoSens'
  viOpenDefaultRM(&rm);

  lStat = viOpen(rm, szResource, 0, 10000, &session);
  if (lStat != VI_SUCCESS)
  {
    fprintf(stderr, "Unknown VISA resource / Device not found '%s'\n", szResource);
    return -2;
}

  viSetAttribute(session, VI_ATTR_TERMCHAR, 0x0a);  // This is important...
  viSetAttribute(session, VI_ATTR_TERMCHAR_EN, 1);  // ...when using SCPI-RAW

  // Information about RemoSens Interface-Box
  // -----------------------------------------------------------------
  memset(szBuf, 0x00, sizeof(szBuf));
  viWrite(session, (ViPBuf)"SYST:IFVersion?\n", 16, &lCount);
  viRead(session, (ViPBuf)szBuf, sizeof(szBuf), &lCount);
  printf("Current 'RemoSens' Interface-Box FW Version: %s", szBuf);
  // -----------------------------------------------------------------

  printf("\nPress <ENTER> to start the Firmware Update...");
  getchar();
  printf("\n");

  FILE *fpIn = 0;
  fpIn = fopen(szUpdateFile, "rb");
  if (!fpIn)
  {
    fprintf(stderr, "Cannot access update file '%s'\n", szUpdateFile);
    return -2;
  }

  long lSizeOfUpdateFile = 0L;
  fseek(fpIn, 0, SEEK_END);
  lSizeOfUpdateFile = ftell(fpIn);
  fseek(fpIn, 0, SEEK_SET);

  char *cpUpdateCmd = new char[lSizeOfUpdateFile + 15]; // +15 because we will prepend
                                                        // "SYST:IFUP " and append "\n"

  // clear buffer
  memset(cpUpdateCmd, 0x00, lSizeOfUpdateFile + 15);
  strcpy(cpUpdateCmd, "SYST:IFUP ");
  fread(cpUpdateCmd + strlen(cpUpdateCmd), sizeof(char), lSizeOfUpdateFile, fpIn);
  strcat(cpUpdateCmd, "\n");
  ViUInt32 lCmdLen = (ViUInt32)strlen(cpUpdateCmd);

  // Data has been read in. Close file
  fclose(fpIn);

  // Send the new firmware to the RemoSens device
  lCount = 0;
  lStat = viWrite(session, (ViPBuf)cpUpdateCmd, lCmdLen, &lCount);

  delete [] cpUpdateCmd;

  Sleep(1000);

  // Since RemoSens will reboot after a firmware-update,
  // the local session will become invalid
  viClose(session);
  viClose(rm);


  // It will take some seconds until the new Firmware is flashed
  // and RemoSens has rebooted itself
  for (int n = iRemainingTime; n > 0; --n)
  {
    printf("RemoSens firmware update in progress - %d s remaining...\n", n);
    Sleep(1000);
  }


  printf("\nPress <ENTER> to finish the application...");
  getchar();
  printf("\n");
  return 0;
}

